﻿Public Class ArrowSource
    Inherits BaseCreature

    Public Enum ArrowSourceType
        ArrowSourceUp = 0
        ArrowSourceDown = 1
        ArrowSourceRight = 2
        ArrowSourceLeft = 3
    End Enum

    Public ObjectName As String = ""                                ' set the default message
    Public DebugRender As Boolean = True                            ' the message should be rendered in debug mode
    Private TextSurface As AgateLib.DisplayLib.FontSurface          ' used to redner the message
    Private MsgBoxWidth As Integer = 0                              ' the width and heigh of the messagebox
    Private MsgBoxHeight As Integer = 0
    Public ArrowSourceDirection As ArrowSourceType

    ' the constructor
    Public Sub New(ByVal AT As ArrowSourceType)
        IsAffactedByArrow = False
        IsAffactedByExplosion = False

        ArrowSourceDirection = AT
        If TextSurface Is Nothing Then TextSurface = New AgateLib.DisplayLib.FontSurface("Times", 10, FontStyle.Bold) ' generate the font surface
        CreatureType = CretureTypeEnum.CreatureType_ArrowSource                                ' type of creature
        HPEffect = 0                                                                    ' has no effect on the player
        IsRendered = True                                                               ' is rednered
        AccX = 0                                                                        ' other setting are similar to other creatures
        AccY = 0
        RenderStage = 0
        CreatureWidth = 16
        CreatureHeight = 16
        HalfWidth = CreatureWidth \ 2
        HalfHeight = CreatureHeight \ 2
        IsAffectedByGravity = False
        CanBounce = False
        Gravity = 0
        MaxGravity = 0
        SolidBlocks = New List(Of MapBlockType)
        StandOnBlocks = New List(Of MapBlockType)

        ' load the sprite
        If AT = ArrowSourceType.ArrowSourceDown Then CreatureSprites = GameSpritesObject.ArrowSourceDownSprite
        If AT = ArrowSourceType.ArrowSourceUp Then CreatureSprites = GameSpritesObject.ArrowSourceUpSprite
        If AT = ArrowSourceType.ArrowSourceLeft Then CreatureSprites = GameSpritesObject.ArrowSourceLeftSprite
        If AT = ArrowSourceType.ArrowSourceRight Then CreatureSprites = GameSpritesObject.ArrowSOurceRightSprite
        CreatureSprites.CurrentFrameIndex = 0

    End Sub

    Public Overrides Sub UpdateLocation(ByVal Info As Object, ByVal UserInput As GameInputClass)
        ' nothing, beacuse it is fixed
        DebugRender = False

    End Sub

    ' rendering is modified 
    Public Overrides Sub RenderCreature()





        ' check to see if it is rednered
        If Not IsRendered Then
            Exit Sub
        End If

        ' if the creature is outside the map, don't render
        If MapX < GameMapObject.ViewX1 Or MapX > GameMapObject.ViewX2 Or MapY < GameMapObject.ViewY1 Or MapY > GameMapObject.ViewY2 Then
            Exit Sub
        End If


        ' render the message
        If DebugRender Then

            ' find the width of the message
            If MsgBoxWidth = 0 Then
                MsgBoxWidth = TextSurface.StringDisplayWidth(Me.ObjectName) + 4
                MsgBoxHeight = TextSurface.StringDisplayHeight(Me.ObjectName) + 4
            End If

            ' find where to place the message
            Dim DisplayX As Long = MapX - GameMapObject.ViewX1 + GameMapObject.ViewXShift - Me.HalfWidth + SpriteXShift - MsgBoxWidth \ 2
            Dim DisplayY As Long = MapY - GameMapObject.ViewY1 + GameMapObject.ViewYShift - Me.CreatureHeight + SpriteYShift - 32 - MsgBoxHeight

            ' draw the pop up msgbox
            AgateLib.DisplayLib.Display.DrawRect(DisplayX, DisplayY, MsgBoxWidth, MsgBoxHeight, AgateLib.Geometry.Color.White)
            AgateLib.DisplayLib.Display.FillRect(DisplayX + 1, DisplayY + 1, MsgBoxWidth - 1, MsgBoxHeight - 1, AgateLib.Geometry.Color.Black)

            ' set the transperency of the message and part to display
            TextSurface.Color = AgateLib.Geometry.Color.FromArgb(255, 255, 255, 255)
            TextSurface.DrawText(DisplayX + 2, DisplayY + 2, Me.ObjectName)
        End If


        ' set render stage 
        CreatureSprites.CurrentFrameIndex = RenderStage

        ' render
        CreatureSprites.Draw(MapX - GameMapObject.ViewX1 + GameMapObject.ViewXShift - Me.HalfWidth + SpriteXShift, MapY - GameMapObject.ViewY1 + GameMapObject.ViewYShift - Me.CreatureHeight + SpriteYShift)
    End Sub
End Class
