﻿' this class describes an object and its behaviour

' define the direction enumeration
Public Enum CreatueXDirection
    LeftDirection = 1
    RightDirection = 2
End Enum

' define different types of creatures that could be represented
Public Enum CretureTypeEnum
    CreatureType_Hero = 0
    CreatureType_Box = 1
    CreatureType_Smoke = 2
    CreatureType_Monkey = 3
    CreatureType_StupidEnemy = 4
    CreatureType_PresistantEnemy = 5
    CreatureType_Info = 6
    CreatureType_SpiderEnemy = 7
    CreatureType_OnOffButton = 8
    CreatureType_Gate = 9
    CreatureType_Action = 10
    CreatureType_WolfmanEnemy = 11
    CreatureType_Teleport = 12
    CreatureType_BatEnemy = 13
    CreatureType_Heroine = 14
    CreatureType_PushButton = 15
    CreatureType_Gate2 = 16
    CreatureType_ArrowButton = 17
    CreatureType_Arrow = 18
    CreatureType_ArrowSource = 19
End Enum

' end specifying the control mode
Public Enum CreatureControlModeEnum
    ControlModePlayer = 0
    ControlModeFriendAI = 1
    ControlModeEnemyAI = 2
    ControlModeFollowPlayer = 3
    ControlModeWaitForPlayer = 4
    ControlModeSwithchedToPlayer = 5
End Enum


' this is the actual class
Public Class BaseCreature

    Public CreatureType As CretureTypeEnum                    ' used to identify the type of the creature
    Public IsRendered As Boolean                              ' used to indicate the rendering option for the creature
    Public IsAffectedByGravity As Boolean                     ' used to indicate how this object is affected by gravity
    Public MapX As Single                                     ' the x location on the map
    Public MapY As Single                                     ' the y location on the map
    Public AccY As Single                                     ' the y acceleration
    Public AccX As Single                                     ' the x acceleration
    Public SolidBlocks As List(Of MapBlockType)           ' the type of objects the creature bounces off
    Public StandOnBlocks As List(Of MapBlockType)         ' the type of objects the creature could stand on later
    Public Direction As CreatueXDirection                     ' indicates the side which the charactor is facing
    Public RenderStage As Integer                             ' indicates the current motion for the object
    Public CreatureWidth As Integer                           ' the width of the creature
    Public CreatureHeight As Integer                          ' the height of the creture
    Public CreatureSprites As AgateLib.Sprites.Sprite         ' the sprites that creates the creature
    Public CanBounce As Boolean                               ' used to define bouncing behaviour for object - for example bouncing bombs or balls
    Public IsStanding As Boolean                              ' used to indicate if the object is standing on something or not
    Public Gravity As Single                                  ' used to indicate the effect of gravity on the object
    Public MaxGravity As Single                               ' used to indicate the max amount of gravity applied
    Public HPEffect As Integer                                ' if the object is a hostile enemy, then this stores how dangerious the effect of that object on your charactor HP
    Public SteppedInWater As Boolean                          ' checkes if the creature is stepping in water, this affacts its speed.
    Public SpriteXShift As Integer = 0                        ' some sprite objects need to be shifted when rendering, this is the x shift
    Public SpriteYShift As Integer = 0                        ' some sprite objects need to be shifted when rendering, this is the y shift
    Public CoveredWithWater As Boolean = False               ' used to indicate if the creature is covered in water - which is different than stepped in water
    Public ControlMode As CreatureControlModeEnum
    Public IsAffactedByArrow As Boolean
    Public IsAffactedByExplosion As Boolean




    Protected HalfWidth As Integer                            ' used to store half the character width
    Protected HalfHeight As Integer                           ' used to store half the character height


    ' the constructor of a creature, usually overridden by the inherting class
    Public Sub New()
        IsAffactedByArrow = False
        IsAffactedByExplosion = False

        IsRendered = True           ' the creature should be rendered
        MapX = 50                   ' the x location
        MapY = 30                   ' the y location
        AccY = 0                    ' the Y acceleration 0 - should have called this speed to be accurate

        ' specify which type of blocks you can walk on
        SolidBlocks = New List(Of MapBlockType)
        StandOnBlocks = New List(Of MapBlockType)
        SolidBlocks.Add(MapBlockType.BrickBlock)          ' can stand/walk on brick
        SolidBlocks.Add(MapBlockType.RocksBlock)              ' can stand/walk on rocks
        SolidBlocks.Add(MapBlockType.GrassBlock)              ' can stand/walk on grass
        SolidBlocks.Add(MapBlockType.TreeBranches)            ' can stand/walk on tree branches

        Direction = CreatueXDirection.RightDirection            ' the direction the creature is looking at
        RenderStage = 0                                         ' current render stage
        CreatureWidth = 16                                      ' the width
        CreatureHeight = 16                                     ' the height
        HalfWidth = CreatureWidth \ 2                           ' half width
        HalfHeight = CreatureHeight \ 2                         ' half height
        IsAffectedByGravity = True                              ' this creature is affacted by gravity

        CanBounce = True                                        ' this creature can bounce
        Gravity = 3                                             ' the gravity affacts this creature by 3 - the weight of the creature
        MaxGravity = 30                                         ' max value for gravity

    End Sub



    ' update location, used to check if the charactor is moving, if its in water, etc
    Public Sub UpdateLocationBase(ByVal Info As Object, ByVal CreatureControlInput As GameInputClass)

        ' check to see if the creature should be rendered, if not exit the subroutine
        If Not IsRendered Then
            Exit Sub
        End If



        ' these are used to calculate the future point to move to.
        Dim TmpX As Double
        Dim TmpY As Double

        ' based on the previous calculation check if the charactor is stepping in water
        If Not SteppedInWater Then
            TmpX = MapX + AccX * GameTimeObject.GameTimeDiff       ' if not calculate x and y location update normally with full 
            TmpY = MapY + AccY * GameTimeObject.GameTimeDiff       '    effect of x and y speed
        Else
            TmpX = MapX + AccX / 2.5 * GameTimeObject.GameTimeDiff ' if not the speed drops to less than half in water
            TmpY = MapY + AccY / 2.5 * GameTimeObject.GameTimeDiff '    so the creature slows down in water
            SteppedInWater = False                  ' set the stepped in water falg to false, it will get updated later
        End If


        ' now based on the side of your character, try to see if there is a tile ahead that is blocking its motion
        Dim LeftEdge As Integer
        Dim RightEdge As Integer
        Dim TopEdge As Integer
        Dim BottomEdge As Integer
        Dim LeftEdge16 As Integer
        Dim RightEdge16 As Integer
        Dim TopEdge16 As Integer
        Dim BottomEdge16 As Integer

        If AccX < 0 Then                                        ' if the character moving to left then check the tiles to left
            LeftEdge = TmpX - HalfWidth                         ' get left edge location
            BottomEdge = TmpY - 1                               ' get buttom edge
            TopEdge = TmpY - CreatureHeight + 1                 ' get the top edge

            TopEdge16 = TopEdge \ 16                            ' now get these edges based on the map tile corrdonates. Each tile 
            BottomEdge16 = BottomEdge \ 16                      '     stands for 16x16 pixel, so divide this by 16
            LeftEdge16 = LeftEdge \ 16

            For I = 0 To SolidBlocks.Count - 1                                ' check every possible tile type that could be blocking you
                For K = TopEdge16 To BottomEdge16                               ' check all the blockes of the right that the character is in touch with
                    If SolidBlocks(I) = GameMapObject.GameMap(LeftEdge16, K) Then
                        TmpX = MapX                                             ' if there is one, then don't update the x location
                        If CanBounce Then
                            If Not IsAffectedByGravity Then
                                AccX = -AccX                                   ' and if the charactor can bounce then its speed is reverse, and reduced  a little bit
                            Else
                                AccX = -AccX / 1.5                                  ' and if the charactor can bounce then its speed is reverse, and reduced  a little bit
                            End If
                        Else
                            AccX = 0                                            ' otherwise just make it stop
                        End If
                        GoTo Out1                                               ' no need for further checks
                    End If
                Next
            Next
Out1:
        ElseIf AccX > 0 Then                                                    ' same as befor but check the other side
            RightEdge = TmpX + HalfWidth
            BottomEdge = TmpY - 1
            TopEdge = TmpY - CreatureHeight + 1

            TopEdge16 = TopEdge \ 16
            BottomEdge16 = BottomEdge \ 16
            RightEdge16 = RightEdge \ 16
            For I = 0 To SolidBlocks.Count - 1
                For K = TopEdge16 To BottomEdge16
                    If SolidBlocks(I) = GameMapObject.GameMap(RightEdge16, K) Then
                        TmpX = MapX
                        If CanBounce Then
                            If Not Me.IsAffectedByGravity Then
                                AccX = -AccX
                            Else
                                AccX = -AccX / 1.5
                            End If
                        Else
                            AccX = 0
                        End If
                        GoTo Out2
                    End If
                Next
            Next
Out2:
        End If

        ' next we are going to check up, and down, so find the edges
        LeftEdge = MapX - HalfWidth                         ' left edge
        RightEdge = MapX + HalfWidth                        ' right edge
        TopEdge = TmpY - CreatureHeight - 1                 ' top edge
        BottomEdge = TmpY + 1                               ' bottom edge

        LeftEdge16 = LeftEdge \ 16                          ' map the edges to map coordinates
        RightEdge16 = RightEdge \ 16
        TopEdge16 = TopEdge \ 16
        BottomEdge16 = BottomEdge \ 16





        ' check to see if the object is within the bounds of the map   -  the 4 tiles are considered as margins, and the 
        ' character is not rendered if it reaches such location.
        If RightEdge16 >= GameMapObject.GameMap.GetLength(0) - 3 Or LeftEdge16 < 3 Then
            IsRendered = False
            Exit Sub
        End If
        If BottomEdge16 >= GameMapObject.GameMap.GetLength(1) - 3 Or TopEdge16 < 3 Then
            IsRendered = False
            Exit Sub
        End If


        ' now is this object affected by gravity? and possibly can bounce?
        If IsAffectedByGravity Then


            ' check to see if it get affacted by water
            Me.CoveredWithWater = False             ' assume it is not covered with water
            For K = LeftEdge16 To RightEdge16       ' check to see if it is stepping in water
                If GameMapObject.GameMap(K, BottomEdge16) = MapBlockType.WaterBlock Or GameMapObject.GameMap(K, TopEdge16) = MapBlockType.WaterBlock Then
                    SteppedInWater = True
                End If
                If GameMapObject.GameMap(K, TopEdge16 + 1) = MapBlockType.WaterBlock Then     ' also check to see if it is covered with water
                    Me.CoveredWithWater = True
                End If
            Next


            ' next check the y acceleration/ or speed to be accurate
            If AccY >= 0 Then
                IsStanding = False

                ' the acc of y get updated differently, check first to see if there is something you can stand on
                Dim I As Integer
                Dim K As Integer
                Dim ProcessedY As Boolean = False
                Dim StepOnSomething As Boolean = False


                For I = 0 To SolidBlocks.Count - 1
                    For K = LeftEdge16 To RightEdge16                                 ' check to see if the character is
                        If SolidBlocks(I) = GameMapObject.GameMap(K, BottomEdge16) Then         ' standing on or touching a platform
                            StepOnSomething = True
                            Exit For
                        End If
                    Next
                    If StepOnSomething Then                                             ' if so, check to see if it could bounce

                        If CanBounce Then
                            ' the object should bounce
                            If AccY > 0 Then
                                AccY = -AccY / 1.5                                      ' if it could bounce then its speed should be reversed and reduced a little
                            End If
                        Else
                            ' the object is standing on a brick or something
                            AccY = 0
                            TmpY = BottomEdge16 * 16                                    ' update the location so that it stands exactly on the platform
                            IsStanding = True
                        End If
                        ProcessedY = True
                        Exit For
                    End If
                Next

                For I = 0 To StandOnBlocks.Count - 1
                    For K = LeftEdge16 To RightEdge16                                 ' check to see if the character is
                        If StandOnBlocks(I) = GameMapObject.GameMap(K, BottomEdge16) Then         ' standing on or touching a platform
                            StepOnSomething = True
                            Exit For
                        End If
                    Next
                    If StepOnSomething Then                                             ' if so, check to see if it could bounce

                        If CanBounce Then
                            ' the object should bounce
                            If AccY > 0 Then
                                AccY = -AccY / 1.5                                      ' if it could bounce then its speed should be reversed and reduced a little
                            End If
                        Else
                            ' the object is standing on a brick or something
                            AccY = 0
                            TmpY = BottomEdge16 * 16                                    ' update the location so that it stands exactly on the platform
                            IsStanding = True
                        End If
                        ProcessedY = True
                        Exit For
                    End If
                Next

                ' if there was no processing, the object must fall
                If Not ProcessedY Then
                    AccY = AccY + Gravity * GameTimeObject.GameTimeDiff                                ' update the speed
                    If AccY > MaxGravity Then                                           ' make sure the speed does not increase forever
                        AccY = MaxGravity
                    End If
                End If


            Else

                IsStanding = False                                                      ' this part where the charactor is jumping

                ' check and see the top
                Dim I As Integer
                Dim ProcessedY As Boolean = False
                Dim SomethingOnTop As Boolean = False
                For I = 0 To SolidBlocks.Count - 1
                    For K = LeftEdge16 To RightEdge16
                        If SolidBlocks(I) = GameMapObject.GameMap(K, TopEdge16) Then
                            SomethingOnTop = True
                            Exit For
                        End If
                    Next

                    If SomethingOnTop Then                                              ' if there is somthing on top then 

                        If CanBounce Then
                            ' the object should bounce
                            If AccY < 0 Then
                                AccY = -AccY / 1.5
                            End If
                        Else
                            ' else the object hit the roof or something, so it must fall
                            AccY = 0
                        End If
                        ProcessedY = True
                        Exit For
                    End If

                Next

                ' if there was no processing the object must fall
                If Not ProcessedY Then
                    AccY = AccY + Gravity * GameTimeObject.GameTimeDiff
                    If AccY > MaxGravity Then
                        AccY = MaxGravity
                    End If
                End If
            End If




            ' if it is not affacted by gravity but can bounce then check things out
        ElseIf CanBounce Then



            ' check to see if it get affacted by water
            Me.CoveredWithWater = False             ' assume it is not covered with water
            For K = LeftEdge16 To RightEdge16       ' check to see if it is stepping in water
                If GameMapObject.GameMap(K, BottomEdge16) = MapBlockType.WaterBlock Or GameMapObject.GameMap(K, TopEdge16) = MapBlockType.WaterBlock Then
                    SteppedInWater = True
                End If
                If GameMapObject.GameMap(K, TopEdge16 + 1) = MapBlockType.WaterBlock Then     ' also check to see if it is covered with water
                    Me.CoveredWithWater = True
                End If
            Next


            ' next check the y acceleration/ or speed to be accurate
            If AccY >= 0 Then
                IsStanding = False

                ' the acc of y get updated differently, check first to see if there is something you can stand on
                Dim I As Integer
                Dim K As Integer
                Dim ProcessedY As Boolean = False
                Dim StepOnSomething As Boolean = False


                For I = 0 To SolidBlocks.Count - 1
                    For K = LeftEdge16 To RightEdge16                                 ' check to see if the character is
                        If SolidBlocks(I) = GameMapObject.GameMap(K, BottomEdge16) Then         ' standing on or touching a platform
                            StepOnSomething = True
                            Exit For
                        End If
                    Next
                    If StepOnSomething Then                                             ' if so, check to see if it could bounce
                        AccY = -AccY                                            ' if it could bounce then its speed should be reversed and reduced a little
                        ProcessedY = True
                        Exit For
                    End If
                Next

                For I = 0 To StandOnBlocks.Count - 1
                    For K = LeftEdge16 To RightEdge16                                 ' check to see if the character is
                        If StandOnBlocks(I) = GameMapObject.GameMap(K, BottomEdge16) Then         ' standing on or touching a platform
                            StepOnSomething = True
                            Exit For
                        End If
                    Next
                    If StepOnSomething Then                                             ' if so, check to see if it could bounce
                        ' the object should bounce
                        AccY = -AccY                                            ' if it could bounce then its speed should be reversed and reduced a little
                        ProcessedY = True
                        Exit For
                    End If
                Next




            Else

                IsStanding = False                                                      ' this part where the charactor is jumping

                ' check and see the top
                Dim I As Integer
                Dim ProcessedY As Boolean = False
                Dim SomethingOnTop As Boolean = False
                For I = 0 To SolidBlocks.Count - 1
                    For K = LeftEdge16 To RightEdge16
                        If SolidBlocks(I) = GameMapObject.GameMap(K, TopEdge16) Then
                            SomethingOnTop = True
                            Exit For
                        End If
                    Next

                    If SomethingOnTop Then                                              ' if there is somthing on top then 
                        ' the object should bounce
                        AccY = -AccY
                        ProcessedY = True
                        Exit For
                    End If

                Next

            End If





        End If


        MapX = TmpX
        MapY = TmpY
    End Sub

    ' used in some cases
    Public Overridable Sub UpdateLocation(ByVal Info As Object, ByVal CreatureControlInput As GameInputClass)
        Me.UpdateLocationBase(Info, CreatureControlInput)
    End Sub


    Public Overridable Sub RenderCreature()

        ' check to see if it is used
        If Not IsRendered Then
            Exit Sub
        End If

        ' check to see if the creature within the bounds of the display , if not no need to render
        If MapX < GameMapObject.ViewX1 Or MapX > GameMapObject.ViewX2 Or MapY < GameMapObject.ViewY1 Or MapY > GameMapObject.ViewY2 Then
            Exit Sub
        End If

        ' render the creature
        CreatureSprites.CurrentFrameIndex = RenderStage
        CreatureSprites.Draw(Me.MapX - GameMapObject.ViewX1 + GameMapObject.ViewXShift - Me.HalfWidth + SpriteXShift, Me.MapY - GameMapObject.ViewY1 + GameMapObject.ViewYShift - Me.CreatureHeight + SpriteYShift)

    End Sub


    ' this function is used to check collision of two objects
    Public Function DoesObjectCollideWithCreature(ByVal ObjX As Single, ByVal ObjY As Single, ByVal ObjWidth As Integer, ByVal ObjHeight As Integer) As Boolean
        Dim X1 As Integer = ObjX - (ObjWidth) / 2
        Dim X2 As Integer = ObjX + (ObjWidth) / 2
        Dim Y1 As Integer = ObjY - (ObjHeight) / 2
        Dim Y2 As Integer = ObjY + (ObjHeight) / 2

        Dim XX1 As Integer = Me.MapX - (Me.CreatureWidth) / 2
        Dim XX2 As Integer = Me.MapX + (Me.CreatureWidth) / 2
        Dim YY1 As Integer = Me.MapY - (Me.CreatureHeight) / 2
        Dim YY2 As Integer = Me.MapY + (Me.CreatureHeight) / 2

        If (X1 >= XX1 And X1 <= XX2) OrElse (X2 >= XX1 And X2 <= XX2) OrElse (X1 >= XX1 And X2 <= XX2) OrElse (XX1 >= X1 And XX2 <= X2) Then
            If (Y1 >= YY1 And Y1 <= YY2) OrElse (Y2 >= YY1 And Y2 <= YY2) OrElse (Y1 >= YY1 And Y2 <= YY2) OrElse (YY1 >= Y1 And YY2 <= Y2) Then
                Return True
            End If
        End If



        Return False
    End Function

End Class
