﻿Public Class Heroine
    Inherits Hero


    Public Sub New()
        IsAffactedByArrow = True
        IsAffactedByExplosion = True

        CreatureType = CretureTypeEnum.CreatureType_Heroine             ' type of creature
        IsRendered = True                                               ' rendered
        MapX = 50                                                       ' default x,y, speed, etc
        MapY = 30
        AccY = 0

        ' specify type of blocks you could stand on
        SolidBlocks = New List(Of MapBlockType)
        StandOnBlocks = New List(Of MapBlockType)
        SolidBlocks.Add(MapBlockType.BrickBlock)
        SolidBlocks.Add(MapBlockType.RocksBlock)
        SolidBlocks.Add(MapBlockType.GrassBlock)
        SolidBlocks.Add(MapBlockType.BreakableBrick)

        Direction = CreatueXDirection.RightDirection                    ' looking to the right
        RenderStage = 0

        CreatureWidth = 18                                              ' the width of the character
        Me.SpriteXShift = -2                                          ' shift by -2

        CreatureHeight = 32                                             ' the height
        HalfWidth = CreatureWidth \ 2
        HalfHeight = CreatureHeight \ 2
        IsAffectedByGravity = True
        CanBounce = False
        Gravity = 0.1
        MaxGravity = 10


        CreatureSprites = GameSpritesObject.HeroineSprite
        CreatureSprites.CurrentFrameIndex = 1

        ' load audio file
        Me.RunningSound = GameSoundsObject.RunningSound
        ControlMode = CreatureControlModeEnum.ControlModeWaitForPlayer
        CanSwitchWithOtherPlayer = False
        Me.HeroAxeSprite = GameSpritesObject.HeroineAxeSprite

    End Sub

    Public Overrides Sub UpdateLocation(ByVal Info As Object, ByVal UserInput As GameInputClass)


        If IsUsingAxe Then
            If System.Environment.TickCount - IsUsingAxeTimeStamp > 3000 Then
                IsUsingAxe = False

                ' also check to see if the map box could be affacted
                Dim MMX As Long = MapX
                Dim MMY As Long = MapY + 8
                If Me.Direction = CreatueXDirection.RightDirection Then
                    MMX = MapX + 8
                Else
                    MMX = MapX - 8
                End If
                MMX = MMX \ 16
                MMY = MMY \ 16
                If GameMapObject.GameMap(MMX, MMY) = MapBlockType.BreakableBrick Then
                    GameMapObject.GameMap(MMX, MMY) = MapBlockType.EmptyBlock
                End If
            Else
                Exit Sub
            End If
        End If


        ' this flag is used to indicate motion
        Dim PlayRuningSoungFlag As Boolean = False

        ' this was added for demo purpose
        If IsInDemoMode Then

            ' if is in demo mode, then set the direction based on speed
            If AccX > 0 Then Me.Direction = CreatueXDirection.RightDirection
            If AccX < 0 Then Me.Direction = CreatueXDirection.LeftDirection
        Else

            If ControlMode = CreatureControlModeEnum.ControlModeFollowPlayer Then
                UpdateLocationByFollow(Info)
            ElseIf ControlMode = CreatureControlModeEnum.ControlModeWaitForPlayer Then
                UpdateLocationByWaitForPlayer(Info)
            ElseIf ControlMode = CreatureControlModeEnum.ControlModePlayer Then
                UpdateLocationByPlayer(Info, UserInput)
            End If
        End If

        ' the unaffacted counter is always updated
        Unaffected = Unaffected - 1
        If Unaffected < 0 Then
            Unaffected = 0
        End If

        Dim OldX = Me.MapX


        If ControlMode <> CreatureControlModeEnum.ControlModeFollowPlayer Then
            UpdateLocationBase(Info, UserInput)

            ' this part is used to select the correct sprite.
            ' check if the char is jumping
            If Not IsStanding Then
                If Direction = CreatueXDirection.LeftDirection Then
                    Me.RenderStage = 7
                Else
                    Me.RenderStage = 3
                End If
            ElseIf AccX <> 0 Then
                Dim STG As Integer = (MapX \ 20) Mod 4
                If STG = 0 Then Me.RenderStage = 2
                If STG = 1 Then Me.RenderStage = 0
                If STG = 2 Then Me.RenderStage = 2
                If STG = 3 Then Me.RenderStage = 1
                If Direction = CreatueXDirection.LeftDirection Then
                    Me.RenderStage += 4
                End If
            Else
                If Direction = CreatueXDirection.LeftDirection Then
                    Me.RenderStage = 6
                Else
                    Me.RenderStage = 2
                End If
            End If
        End If

        ' check if the sound should be played
        If OldX <> Me.MapX And IsStanding Then
            PlayRuningSoungFlag = True
        End If

        ' next based on the motion flag check the sound
        If PlayRuningSoungFlag And Not RunningSound.IsPlaying Then
            RunningSound.Play()
        ElseIf Not PlayRuningSoungFlag And RunningSound.IsPlaying Then
            RunningSound.Stop()
        End If


    End Sub


    Private Sub UpdateLocationByPlayer(ByVal Info As Object, ByVal UserINPUT As GameInputClass)

        If CanSwitchWithOtherPlayer AndAlso AgateLib.InputLib.Keyboard.Keys(AgateLib.InputLib.KeyCode.D4) Then
            If Info IsNot Nothing Then
                Dim HR As Hero = Info
                HR.ControlMode = CreatureControlModeEnum.ControlModePlayer
                Me.ControlMode = CreatureControlModeEnum.ControlModeSwithchedToPlayer
            End If
        End If


        ' check to see if the character is covered with water
        If Me.CoveredWithWater Then
            Me.Air = 30000 - (System.Environment.TickCount - Me.AirCounter)
            If Me.Air <= 0 Then
                Me.CurrentHP = Me.CurrentHP - 5
            End If
        Else
            Me.Air = 30000
            Me.AirCounter = System.Environment.TickCount
        End If


        ' the the up button is presses
        If UserINPUT.IsUp Then

            ' check and make sure that the person is standing on a brick
            If IsStanding Then
                AccY = -3.1
            End If
        End If

        ' the accx is affacted by keyboard input only
        AccX = 0

        ' the the right button is presses
        If UserINPUT.IsRight Then
            AccX = +3
            If HasShoe Then
                AccX += 1
            End If
            Me.Direction = CreatueXDirection.RightDirection
        End If

        ' the the left button is presses
        If UserINPUT.IsLeft Then
            AccX = -3
            If HasShoe Then
                AccX -= 1
            End If
            Me.Direction = CreatueXDirection.LeftDirection
        End If

        ' the current weapon is dynamite
        If UserINPUT.Is1 Then
            Me.CurrentWeapon = Box.BoxContent.Dynamite
        End If

        ' the current weapon is bouncing bomb
        If UserINPUT.Is2 Then
            Me.CurrentWeapon = Box.BoxContent.BounceingBomb
        End If

        If ControlMode = CreatureControlModeEnum.ControlModePlayer AndAlso _
           UserInput.Is6 Then
            Me.CurrentWeapon = Box.BoxContent.Axe
        End If

    End Sub

    Private Sub UpdateLocationByFollow(ByVal Info As Object)
        Dim H As Hero = Info
        Dim Index As Long = H.HistoryQueueIndex - 100
        If Index < 0 Then Index = Index + H.HistoryQueue.Count

        Dim Obj = H.HistoryQueue(Index)

        Me.MapX = Obj.MapX
        Me.MapY = Obj.MapY
        Me.IsStanding = Obj.IsStanding
        Me.RenderStage = Obj.RenderStage
    End Sub

    Private Sub UpdateLocationByWaitForPlayer(ByVal Info As Object)
        If Info IsNot Nothing Then
            Dim HR As Hero = Info
            If HR.DoesObjectCollideWithCreature(Me.MapX, Me.MapY, Me.CreatureWidth, Me.CreatureHeight) Then
                Me.ControlMode = CreatureControlModeEnum.ControlModeFollowPlayer
                For Each itm In HR.HistoryQueue
                    itm.MapX = Me.MapX
                    itm.MapY = Me.MapY
                    itm.IsStanding = Me.IsStanding
                    itm.RenderStage = Me.RenderStage
                Next
                CanSwitchWithOtherPlayer = True
                HR.CanSwitchWithOtherPlayer = True
            End If
        End If
    End Sub

    Public Overrides Sub RenderCreature()
        If Unaffected > 0 Then
            If Math.Round(Unaffected) Mod 2 = 0 Then
                Exit Sub
            End If
        End If

        MyBase.RenderCreature()
    End Sub


End Class
