﻿' define the block type
Public Enum MapBlockType
    EmptyBlock = 0
    BrickBlock = 1
    GrassBlock = 2
    WaterBlock = 3
    RocksBlock = 4
    Grass = 5
    NotAssigned2 = 6
    leaves = 7
    TreeTrank = 8
    TreeBranches = 9
    NotAssigned4 = 10
    FullTree = 11
    FullTree2 = 12
    BrokenSpaceShip = 13
    Hut = 14
    SpaceShip = 15
    Window6x6 = 16
    GrayDoor = 17
    Brick6x6 = 18
    StreetLamp = 19
    Window6x6Transperent = 20
    GrayDoor_2 = 21
    Brick6x6_2 = 22
    Window6x6_2 = 23
    Brick6x6_3 = 24
    SmallBackgroundBrick_1 = 25
    SmallBackgroundBrick_2 = 26
    SmallBackgroundBrick_3 = 27
    BreakableBrick = 28
End Enum

Public Class GameMapClass


    ' <<****************************************************************************



    ' >>****************************************************************************


    ' <<****************************************************************************
    ' define wallpaper class 
    Public Class WallpaperClass
        Public WallpaperImg As AgateLib.DisplayLib.Surface
        Public UseWallpaper As Boolean = False
        Public WallpaperWidth As Integer = 0
        Public WallpaperHeight As Integer = 0
        Public WallpaperXRatio As Double
        Public WallpaperYRatio As Double
    End Class
    ' >>****************************************************************************


    ' the wallpaper
    Public Wallpaper As New WallpaperClass

    ' define the array of the map
    Public GameMap(,) As MapBlockType

    ' define the shapes of each block
    Public BrickShapes As New List(Of AgateLib.DisplayLib.Surface)

    ' the water motion blocks
    Public WaterMotion As New List(Of AgateLib.DisplayLib.Surface)

    ' the street lamp
    Public StreetLamp As AgateLib.DisplayLib.Surface

    ' this is the current map song
    Public MapMusic As Microsoft.DirectX.AudioVideoPlayback.Audio
    Public MapMusicPosition As Double = -1

    ' vars used by other modules
    Public ViewX1 As Integer
    Public ViewX2 As Integer
    Public ViewY1 As Integer
    Public ViewY2 As Integer
    Public ViewXShift As Integer
    Public ViewYShift As Integer

    Public GameMapActualWidth As Long
    Public GameMapActualHeight As Long
    Public GameMapWidthDiv16 As Long
    Public GameMapHeightDiv16 As Long


    ' this sub will be used to create the brick shapes/ load them
    Public Sub CreateBrickShapes()
        BrickShapes.Clear()
        BrickShapes.Add(Nothing)
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\brickblock.bmp"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\grassblock.bmp"))
        BrickShapes.Add(Nothing)
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\rocksblock.bmp"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\Grass.png"))
        BrickShapes.Add(Nothing)
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\leaves.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\treetrank.bmp"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\treebranches.png"))
        BrickShapes.Add(Nothing)
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\fulltree.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\fulltree2.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\brokenspaceship.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\hut.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\spaceship.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\Window6x6.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\graydoor.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\Brick6x6.png"))
        BrickShapes.Add(Nothing)
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\window6x6 transperent.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\graydoor-2.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\Brick6x6-2.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\window6x6-2.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\Brick6x6-3.png"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\SmallBackgroundBrick-1.bmp"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\SmallBackgroundBrick-2.bmp"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\SmallBackgroundBrick-3.bmp"))
        BrickShapes.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\breakable-brick.bmp"))


        ' load water motion
        Dim I As Integer
        For I = 1 To 16
            WaterMotion.Add(New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\waterblock" & I & ".bmp"))
            WaterMotion(I - 1).Alpha = 0.75
        Next

        ' load the street lamp
        StreetLamp = New AgateLib.DisplayLib.Surface(Application.StartupPath & "\images\streetlamp.png")

    End Sub


    ' used to play music
    Public Sub PlayMapMusic()
        If MapMusic IsNot Nothing Then
            If MapMusic.CurrentPosition = Me.MapMusicPosition Then

                MapMusic.SeekCurrentPosition(0, Microsoft.DirectX.AudioVideoPlayback.SeekPositionFlags.AbsolutePositioning)
                MapMusic.Play()

            End If
            MapMusicPosition = Me.MapMusic.CurrentPosition
        End If

    End Sub

    ' next use this function to generate a random map
    Public Sub CreateRandomMap(ByVal MapWidth As Integer, ByVal MapHeight As Integer, ByVal MaxRockLayers As Integer, ByVal MaxAppleTrees As Integer, ByVal MaxWaterPonds As Integer)

        CreateEmptyMap(MapWidth, MapHeight)
        Dim R As Random = GameRandomNumberObject
        Dim I As Integer
        Dim J As Integer
        Dim K As Integer


        ' put rocks

        For K = 0 To MaxRockLayers - 1

            For I = 1 To MapWidth - 2
                GameMap(I, MapHeight - 1) = MapBlockType.RocksBlock
                If R.Next(0, 100) < 90 Then

                    For J = MapHeight - 1 To 1 Step -1
                        If (GameMap(I, J - 1) <> MapBlockType.RocksBlock And GameMap(I, J) = MapBlockType.RocksBlock) Then
                            If GameMap(I - 1, J) = MapBlockType.RocksBlock And GameMap(I + 1, J) = MapBlockType.RocksBlock Then
                                GameMap(I, J - 1) = MapBlockType.RocksBlock
                            End If
                            Exit For
                        End If
                    Next

                End If
            Next

        Next

        ' put grass
        For I = 0 To MapWidth - 1

            For J = MapHeight - 1 To 1 Step -1
                If (GameMap(I, J - 1) <> MapBlockType.RocksBlock And GameMap(I, J) = MapBlockType.RocksBlock) Then
                    GameMap(I, J - 1) = MapBlockType.Grass
                    Exit For
                End If
            Next

        Next


        ' put water ponds
        For I = 0 To MaxWaterPonds - 1

            ' first get the top size of the pond
            Dim BS As Integer = R.Next(3, 10)

            ' start searching for a place where there are rocks equale to bs
            Dim Loc As Integer = R.Next(10, MapWidth - 1 - 10)

            ' now break the blocks from top to bottom until you read a place where an entire row is made of bricks
            Dim Top As Integer
            For J = 0 To MapHeight - 1
                Dim B As Boolean = True
                Top = J
                For K = 0 To BS - 1
                    If GameMap(Loc + K, J) <> MapBlockType.RocksBlock Then
                        B = False
                    End If
                    GameMap(Loc + K, J) = MapBlockType.EmptyBlock
                Next
                If B Then
                    Exit For
                End If
            Next

            Do
                If Top = MapHeight - 1 Then
                    Exit For
                End If
                For J = 0 To BS - 1
                    GameMap(Loc + J, Top) = MapBlockType.WaterBlock
                Next
                Top = Top + 1
                K = R.Next(0, BS)
                Loc += K
                BS -= K
                BS = R.Next(0, BS)
                If BS < 2 Then
                    Exit Do
                End If
            Loop
        Next

        ' next perform serveral passes on water to see if it needs flatening
        Dim FLG As Boolean
        Dim Bottom As Integer
        Do
            FLG = False
            For I = 3 To MapWidth - 4
                Bottom = -1
                For J = MapHeight - 1 To 3 Step -1
                    If GameMap(I, J) = MapBlockType.WaterBlock Then
                        Bottom = J
                        Exit For
                    End If
                Next
                If Bottom <> -1 Then
                    If GameMap(I - 1, Bottom) = MapBlockType.RocksBlock And GameMap(I - 1, Bottom - 1) = MapBlockType.RocksBlock Then

                        FLG = True

                        For K = Bottom - 1 To 0 Step -1
                            If GameMap(I, K) = MapBlockType.WaterBlock Then
                                GameMap(I - 1, K) = MapBlockType.WaterBlock
                            Else
                                GameMap(I - 1, K) = MapBlockType.EmptyBlock
                            End If
                        Next

                        Exit For

                    End If
                End If
            Next
        Loop While FLG

        Do
            FLG = False
            For I = MapWidth - 4 To 3 Step -1
                Bottom = -1
                For J = MapHeight - 1 To 3 Step -1
                    If GameMap(I, J) = MapBlockType.WaterBlock Then
                        Bottom = J
                        Exit For
                    End If
                Next
                If Bottom <> -1 Then
                    If GameMap(I + 1, Bottom) = MapBlockType.RocksBlock And GameMap(I + 1, Bottom - 1) = MapBlockType.RocksBlock Then

                        FLG = True

                        For K = Bottom - 1 To 0 Step -1
                            If GameMap(I, K) = MapBlockType.WaterBlock Then
                                GameMap(I + 1, K) = MapBlockType.WaterBlock
                            Else
                                GameMap(I + 1, K) = MapBlockType.EmptyBlock
                            End If
                        Next

                        Exit For

                    End If
                End If
            Next
        Loop While FLG

        ' put some trees
        For I = 0 To MaxAppleTrees - 1

            Do
                Dim X As Integer = R.Next(0, MapWidth - 20)
                Dim XX As Integer = R.Next(0, 100) Mod 2
                If XX = 0 Then
                    For J = 0 To MapHeight - 11
                        If GameMap(X + 3, J + 10) = MapBlockType.RocksBlock And GameMap(X + 4, J + 10) = MapBlockType.RocksBlock Then
                            GameMap(X, J) = MapBlockType.FullTree
                            Exit Do
                        End If
                    Next
                Else
                    For J = 0 To MapHeight - 20
                        If GameMap(X + 16, J + 19) = MapBlockType.RocksBlock And GameMap(X + 17, J + 19) = MapBlockType.RocksBlock Then
                            GameMap(X, J) = MapBlockType.FullTree2
                            Exit Do
                        End If
                    Next
                End If

            Loop

        Next


    End Sub

    ' use this to render the map
    Public Sub RenderMap(ByVal RenderAll As Boolean)


        ' update the water effect
        Dim No = (System.Environment.TickCount Mod 1000) / 1000.0 * 15
        BrickShapes(3) = WaterMotion(No)
        BrickShapes(19) = StreetLamp
        If Not RenderAll Then
            BrickShapes(3) = Nothing
            BrickShapes(19) = Nothing
        End If

        ' the location where the plotting starts
        ViewX1 = GameDisplayObject.EyeX - GameDisplayObject.MidDisplayX - 16
        ViewX2 = GameDisplayObject.EyeX + GameDisplayObject.MidDisplayX + 16

        If ViewX1 >= 0 And ViewX2 <= Me.GameMapActualWidth Then
            ' x can be plotted without a problem
            ViewXShift = 0
        ElseIf ViewX1 < 0 Then
            ' shift the x
            ViewX1 = 0
            ViewX2 = GameDisplayObject.MyDisplayWidth
            If ViewX2 > me.gameMapActualWidth Then
                ViewX2 = Me.GameMapActualWidth
                ViewXShift = (GameDisplayObject.MyDisplayWidth - Me.GameMapActualWidth) / 2
            Else
                ViewXShift = 0
            End If
        ElseIf ViewX2 > Me.GameMapActualWidth Then
            ViewX2 = Me.GameMapActualWidth
            ViewX1 = Me.GameMapActualWidth - GameDisplayObject.MyDisplayWidth
            If ViewX1 < 0 Then
                ViewX1 = 0
                ViewXShift = (GameDisplayObject.MyDisplayWidth - Me.GameMapActualWidth) / 2
            Else
                ViewXShift = 0
            End If
        End If

        ViewY1 = GameDisplayObject.EyeY - GameDisplayObject.MidDisplayY - 16
        ViewY2 = GameDisplayObject.EyeY + GameDisplayObject.MidDisplayY + 16

        If ViewY1 >= 0 And ViewY2 <= Me.GameMapActualHeight Then
            ' x can be plotted without a problem
            ViewYShift = 0
        ElseIf ViewY1 < 0 Then
            ' shift the x
            ViewY1 = 0
            ViewY2 = GameDisplayObject.MyDisplayHeight
            If ViewY2 > Me.GameMapActualHeight Then
                ViewY2 = Me.GameMapActualHeight
                ViewYShift = (GameDisplayObject.MyDisplayHeight - Me.GameMapActualHeight) / 2
            Else
                ViewYShift = 0
            End If
        ElseIf ViewY2 > Me.GameMapActualHeight Then
            ViewY2 = Me.GameMapActualHeight
            ViewY1 = Me.GameMapActualHeight - GameDisplayObject.MyDisplayHeight
            If ViewY1 < 0 Then
                ViewY1 = 0
                ViewYShift = (GameDisplayObject.MyDisplayHeight - Me.GameMapActualHeight) / 2
            Else
                ViewYShift = 0
            End If
        End If

        Dim SMX1 As Integer
        Dim SMX2 As Integer
        Dim SMY1 As Integer
        Dim SMY2 As Integer
        Dim SMXX As Integer
        Dim SMYY As Integer


        SMX1 = ViewX1 \ 16
        SMX2 = ViewX2 \ 16 - 1
        SMY1 = ViewY1 \ 16
        SMY2 = ViewY2 \ 16 - 1
        SMXX = 16 - ViewX1 Mod 16
        SMYY = 16 - ViewY1 Mod 16

        If SMX2 >= GameMap.GetLength(0) Then
            SMX2 = GameMap.GetLength(0) - 1
        End If
        If SMY2 >= GameMap.GetLength(1) Then
            SMY2 = GameMap.GetLength(1) - 1
        End If

        Dim I As Integer
        Dim J As Integer

        ' next render the wallpaper
        If Wallpaper.UseWallpaper Then

            ' calculate where to plot the wallpaper
            Dim WallpaperX As Integer = ViewX1 * Wallpaper.WallpaperXRatio
            Dim WallpaperY As Integer = ViewY1 * Wallpaper.WallpaperYRatio

            ' plot the wallpaper
            Dim SRC As New AgateLib.Geometry.Rectangle(WallpaperX, WallpaperY, GameDisplayObject.MyDisplayWidth, GameDisplayObject.MyDisplayHeight)
            Wallpaper.WallpaperImg.Draw(0, 0, SRC, 0, 0)

            ' continue with plotting the tiles
            With BrickShapes
                For I = SMX1 - 8 To SMX2
                    For J = SMY1 - 10 To SMY2
                        Try
                            If I >= 0 AndAlso J >= 0 AndAlso .Item(GameMap(I, J)) IsNot Nothing Then .Item(GameMap(I, J)).Draw(SMXX + ViewXShift + (I - SMX1) * 16 - 16, SMYY + ViewYShift + (J - SMY1) * 16 - 16)

                        Catch ex As Exception
                            MsgBox(ex.Message)
                        End Try
                    Next
                Next
            End With
        Else
            With BrickShapes
                For I = SMX1 - 8 To SMX2
                    For J = SMY1 - 10 To SMY2
                        If I >= 0 AndAlso J >= 0 AndAlso .Item(GameMap(I, J)) IsNot Nothing Then .Item(GameMap(I, J)).Draw(SMXX + ViewXShift + (I - SMX1) * 16 - 16, SMYY + ViewYShift + (J - SMY1) * 16 - 16)
                    Next
                Next
            End With
        End If

    End Sub

    ' use this to render the water only
    Public Sub RenderWaterAndStreetLampInMap()


        ' update the water effect
        Dim No = (System.Environment.TickCount Mod 1000) / 1000.0 * 15
        BrickShapes(3) = WaterMotion(No)

        ' the location where the plotting starts
        ViewX1 = GameDisplayObject.EyeX - GameDisplayObject.MidDisplayX - 16
        ViewX2 = GameDisplayObject.EyeX + GameDisplayObject.MidDisplayX + 16

        If ViewX1 >= 0 And ViewX2 <= GameMapObject.GameMapActualWidth Then
            ' x can be plotted without a problem
            ViewXShift = 0
        ElseIf ViewX1 < 0 Then
            ' shift the x
            ViewX1 = 0
            ViewX2 = GameDisplayObject.MyDisplayWidth
            If ViewX2 > Me.GameMapActualWidth Then
                ViewX2 = Me.GameMapActualWidth
                ViewXShift = (GameDisplayObject.MyDisplayWidth - Me.GameMapActualWidth) / 2
            Else
                ViewXShift = 0
            End If
        ElseIf ViewX2 > Me.GameMapActualWidth Then
            ViewX2 = Me.GameMapActualWidth
            ViewX1 = Me.GameMapActualWidth - GameDisplayObject.MyDisplayWidth
            If ViewX1 < 0 Then
                ViewX1 = 0
                ViewXShift = (GameDisplayObject.MyDisplayWidth - Me.GameMapActualWidth) / 2
            Else
                ViewXShift = 0
            End If
        End If

        ViewY1 = GameDisplayObject.EyeY - GameDisplayObject.MidDisplayY - 16
        ViewY2 = GameDisplayObject.EyeY + GameDisplayObject.MidDisplayY + 16

        If ViewY1 >= 0 And ViewY2 <= Me.GameMapActualHeight Then
            ' x can be plotted without a problem
            ViewYShift = 0
        ElseIf ViewY1 < 0 Then
            ' shift the x
            ViewY1 = 0
            ViewY2 = GameDisplayObject.MyDisplayHeight
            If ViewY2 > Me.GameMapActualHeight Then
                ViewY2 = Me.GameMapActualHeight
                ViewYShift = (GameDisplayObject.MyDisplayHeight - Me.GameMapActualHeight) / 2
            Else
                ViewYShift = 0
            End If
        ElseIf ViewY2 > Me.GameMapActualHeight Then
            ViewY2 = Me.GameMapActualHeight
            ViewY1 = Me.GameMapActualHeight - GameDisplayObject.MyDisplayHeight
            If ViewY1 < 0 Then
                ViewY1 = 0
                ViewYShift = (GameDisplayObject.MyDisplayHeight - GameMapObject.GameMapActualHeight) / 2
            Else
                ViewYShift = 0
            End If
        End If

        Dim SMX1 As Integer
        Dim SMX2 As Integer
        Dim SMY1 As Integer
        Dim SMY2 As Integer
        Dim SMXX As Integer
        Dim SMYY As Integer


        SMX1 = ViewX1 \ 16
        SMX2 = ViewX2 \ 16 - 1
        SMY1 = ViewY1 \ 16
        SMY2 = ViewY2 \ 16 - 1
        SMXX = 16 - ViewX1 Mod 16
        SMYY = 16 - ViewY1 Mod 16
        If SMX2 >= GameMap.GetLength(0) Then
            SMX2 = GameMap.GetLength(0) - 1
        End If
        If SMY2 >= GameMap.GetLength(1) Then
            SMY2 = GameMap.GetLength(1) - 1
        End If

        Dim I As Integer
        Dim J As Integer

        With BrickShapes
            For I = SMX1 - 8 To SMX2
                For J = SMY1 - 10 To SMY2
                    If I >= 0 AndAlso J >= 0 Then
                        If GameMap(I, J) = MapBlockType.StreetLamp Then
                            StreetLamp.Draw(SMXX + ViewXShift + (I - SMX1) * 16 - 16, SMYY + ViewYShift + (J - SMY1) * 16 - 16)
                        ElseIf GameMap(I, J) = MapBlockType.WaterBlock Then
                            .Item(GameMap(I, J)).Draw(SMXX + ViewXShift + (I - SMX1) * 16 - 16, SMYY + ViewYShift + (J - SMY1) * 16 - 16)
                        End If
                    End If
                Next
            Next
        End With


    End Sub


    ' next use this function to generate an empaty map
    Public Sub CreateEmptyMap(ByVal MapWidth As Integer, ByVal MapHeight As Integer)
        ' resize the array
        ReDim GameMap(0 To MapWidth - 1, 0 To MapHeight - 1)
        GameMapActualWidth = GameMap.GetLength(0) * 16
        GameMapActualHeight = GameMap.GetLength(1) * 16
        GameMapWidthDiv16 = GameMap.GetLength(0)
        GameMapHeightDiv16 = GameMap.GetLength(1)

        ' define the values
        Dim I As Integer
        Dim J As Integer
        For I = 0 To MapWidth - 1
            For J = 0 To MapHeight - 1
                GameMap(I, J) = MapBlockType.EmptyBlock
            Next
        Next

        For I = 0 To MapWidth - 1
            GameMap(I, 3) = MapBlockType.BrickBlock
            GameMap(I, MapHeight - 4) = MapBlockType.BrickBlock
        Next

        For I = 0 To MapHeight - 1
            GameMap(3, I) = MapBlockType.BrickBlock
            GameMap(MapWidth - 4, I) = MapBlockType.BrickBlock
        Next

    End Sub

    ' used to save the map
    Public Sub SaveMap(ByVal FileName As String, ByVal HeroChar1 As BaseCreature, ByVal HeroChar2 As BaseCreature, ByVal EnemyList As List(Of BaseCreature), ByVal BoxList As List(Of Box))
        Try

            Dim FS = System.IO.File.Create(FileName)
            FS.WriteByte(GameMap.GetLength(0) Mod 256)
            FS.WriteByte(GameMap.GetLength(0) \ 256)
            FS.WriteByte(GameMap.GetLength(1) Mod 256)
            FS.WriteByte(GameMap.GetLength(1) \ 256)
            Dim I As Integer
            Dim J As Integer
            For I = 0 To GameMap.GetLength(1) - 1
                For J = 0 To GameMap.GetLength(0) - 1
                    FS.WriteByte(GameMap(J, I))
                Next
            Next

            If HeroChar1 IsNot Nothing AndAlso HeroChar1.IsRendered Then
                FS.WriteByte(255)
                FS.WriteByte(HeroChar1.MapX Mod 256)
                FS.WriteByte(HeroChar1.MapX \ 256)
                FS.WriteByte(HeroChar1.MapY Mod 256)
                FS.WriteByte(HeroChar1.MapY \ 256)
            Else
                FS.WriteByte(0)
            End If
            If HeroChar2 IsNot Nothing AndAlso HeroChar2.IsRendered Then
                FS.WriteByte(255)
                FS.WriteByte(HeroChar2.MapX Mod 256)
                FS.WriteByte(HeroChar2.MapX \ 256)
                FS.WriteByte(HeroChar2.MapY Mod 256)
                FS.WriteByte(HeroChar2.MapY \ 256)
            Else
                FS.WriteByte(0)
            End If

            ' write enemy information
            FS.WriteByte(EnemyList.Count Mod 256)
            FS.WriteByte(EnemyList.Count \ 256)
            For Each Itm As BaseCreature In EnemyList
                If Itm.CreatureType = CretureTypeEnum.CreatureType_StupidEnemy Then FS.WriteByte(0)
                If Itm.CreatureType = CretureTypeEnum.CreatureType_PresistantEnemy Then FS.WriteByte(1)
                If Itm.CreatureType = CretureTypeEnum.CreatureType_Monkey Then FS.WriteByte(2)
                If Itm.CreatureType = CretureTypeEnum.CreatureType_Smoke Then FS.WriteByte(3)
                If Itm.CreatureType = CretureTypeEnum.CreatureType_Info Then
                    FS.WriteByte(4)
                    Dim Tmp As Info = Itm
                    Dim B() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.Message)
                    FS.WriteByte(B.Length Mod 256)
                    FS.WriteByte(B.Length \ 256)
                    FS.Write(B, 0, B.Length)
                End If
                If Itm.CreatureType = CretureTypeEnum.CreatureType_SpiderEnemy Then FS.WriteByte(5)
                If Itm.CreatureType = CretureTypeEnum.CreatureType_OnOffButton Then
                    FS.WriteByte(6)
                    Dim Tmp As OnOffButton = Itm
                    Dim B() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.ObjectToAffact)
                    FS.WriteByte(B.Length Mod 256)
                    FS.WriteByte(B.Length \ 256)
                    FS.Write(B, 0, B.Length)
                End If
                If Itm.CreatureType = CretureTypeEnum.CreatureType_Gate Then
                    FS.WriteByte(7)
                    Dim Tmp As Gate = Itm
                    Dim B() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.ObjectName)
                    FS.WriteByte(B.Length Mod 256)
                    FS.WriteByte(B.Length \ 256)
                    FS.Write(B, 0, B.Length)
                End If
                If Itm.CreatureType = CretureTypeEnum.CreatureType_Action Then
                    FS.WriteByte(8)
                    Dim Tmp As Action = Itm
                    Dim B() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.ActionType)
                    FS.WriteByte(B.Length Mod 256)
                    FS.WriteByte(B.Length \ 256)
                    FS.Write(B, 0, B.Length)
                End If
                If Itm.CreatureType = CretureTypeEnum.CreatureType_WolfmanEnemy Then FS.WriteByte(9)
                If Itm.CreatureType = CretureTypeEnum.CreatureType_Teleport Then
                    FS.WriteByte(10)
                    Dim Tmp As Teleport = Itm
                    Dim B() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.ObjectName)
                    FS.WriteByte(B.Length Mod 256)
                    FS.WriteByte(B.Length \ 256)
                    FS.Write(B, 0, B.Length)
                    Dim B2() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.TeleportTo)
                    FS.WriteByte(B2.Length Mod 256)
                    FS.WriteByte(B2.Length \ 256)
                    FS.Write(B2, 0, B2.Length)
                End If
                If Itm.CreatureType = CretureTypeEnum.CreatureType_BatEnemy Then FS.WriteByte(11)
                If Itm.CreatureType = CretureTypeEnum.CreatureType_PushButton Then
                    FS.WriteByte(12)
                    Dim Tmp As PushButton = Itm
                    Dim B() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.ObjectToAffact)
                    FS.WriteByte(B.Length Mod 256)
                    FS.WriteByte(B.Length \ 256)
                    FS.Write(B, 0, B.Length)
                End If
                If Itm.CreatureType = CretureTypeEnum.CreatureType_Gate2 Then
                    FS.WriteByte(13)
                    Dim Tmp As Gate2 = Itm
                    Dim B() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.ObjectName)
                    FS.WriteByte(B.Length Mod 256)
                    FS.WriteByte(B.Length \ 256)
                    FS.Write(B, 0, B.Length)
                End If
                If Itm.CreatureType = CretureTypeEnum.CreatureType_ArrowButton Then
                    FS.WriteByte(14)
                    Dim Tmp As ArrowButton = Itm
                    Dim B() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.ObjectToAffact)
                    FS.WriteByte(B.Length Mod 256)
                    FS.WriteByte(B.Length \ 256)
                    FS.Write(B, 0, B.Length)
                End If
                If Itm.CreatureType = CretureTypeEnum.CreatureType_ArrowSource Then
                    FS.WriteByte(15)
                    Dim Tmp As ArrowSource = Itm
                    FS.WriteByte(Tmp.ArrowSourceDirection)
                    Dim B() As Byte = System.Text.Encoding.GetEncoding("windows-1256").GetBytes(Tmp.ObjectName)
                    FS.WriteByte(B.Length Mod 256)
                    FS.WriteByte(B.Length \ 256)
                    FS.Write(B, 0, B.Length)
                End If



                FS.WriteByte(Itm.MapX Mod 256)
                FS.WriteByte(Itm.MapX \ 256)
                FS.WriteByte(Itm.MapY Mod 256)
                FS.WriteByte(Itm.MapY \ 256)

            Next


            ' write the boxes info
            FS.WriteByte(BoxList.Count)

            For Each Itm As Box In BoxList
                FS.WriteByte(Itm.ItemType)
                FS.WriteByte(Itm.MapX Mod 256)
                FS.WriteByte(Itm.MapX \ 256)
                FS.WriteByte(Itm.MapY Mod 256)
                FS.WriteByte(Itm.MapY \ 256)
            Next

            FS.Close()

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    ' used to load the map
    Public Sub LoadMap(ByVal FileName As String, ByVal RenderProgress As Boolean)
        Try
            ' clear old objects
            GameCreaturesObject.ClearAll()

            ' open the file
            Dim FS = System.IO.File.Open(FileName, IO.FileMode.Open)

            ' get the width and the height
            Dim W As Integer
            Dim H As Integer
            W = FS.ReadByte + FS.ReadByte * 256
            H = FS.ReadByte + FS.ReadByte * 256

            ' create new map
            Dim M(0 To W - 1, 0 To H - 1) As MapBlockType
            Dim I As Integer
            Dim J As Integer

            ' show messaged
            If RenderProgress Then
                AgateLib.DisplayLib.Display.BeginFrame()
                AgateLib.DisplayLib.Display.Clear(0, 0, 0, 0)
                GameTextSufaceObject.DrawText(0, 0, "1 : loading map")
                AgateLib.DisplayLib.Display.EndFrame()
            End If

            ' load the map
            For I = 0 To M.GetLength(1) - 1
                For J = 0 To M.GetLength(0) - 1
                    M(J, I) = FS.ReadByte
                Next
            Next

            ' show message
            If RenderProgress Then
                AgateLib.DisplayLib.Display.BeginFrame()
                AgateLib.DisplayLib.Display.Clear(0, 0, 0, 0)
                GameTextSufaceObject.DrawText(0, 0, "2 : loading hero")
                AgateLib.DisplayLib.Display.EndFrame()
            End If

            ' load the hero
            If FS.ReadByte = 255 Then
                GameCreaturesObject.UserCharacter1 = New Hero
                GameCreaturesObject.UserCharacter1.IsRendered = True
                GameCreaturesObject.UserCharacter1.MapX = FS.ReadByte + FS.ReadByte * 256
                GameCreaturesObject.UserCharacter1.MapY = FS.ReadByte + FS.ReadByte * 256
            End If

            ' render progress
            If RenderProgress Then
                AgateLib.DisplayLib.Display.BeginFrame()
                AgateLib.DisplayLib.Display.Clear(0, 0, 0, 0)
                GameTextSufaceObject.DrawText(0, 0, "3 : loading heroine")
                AgateLib.DisplayLib.Display.EndFrame()
            End If

            ' load the heoine
            If FS.ReadByte = 255 Then
                GameCreaturesObject.UserCharacter2 = New Heroine
                GameCreaturesObject.UserCharacter2.IsRendered = True
                GameCreaturesObject.UserCharacter2.MapX = FS.ReadByte + FS.ReadByte * 256
                GameCreaturesObject.UserCharacter2.MapY = FS.ReadByte + FS.ReadByte * 256
            End If

            ' get the enemy count
            Dim EnemyCount As Integer = FS.ReadByte + FS.ReadByte * 256

            ' load the enemies
            For I = 0 To EnemyCount - 1

                ' render the progress
                If RenderProgress Then
                    AgateLib.DisplayLib.Display.BeginFrame()
                    AgateLib.DisplayLib.Display.Clear(0, 0, 0, 0)
                    GameTextSufaceObject.DrawText(0, 0, "4 : loading creature no:" & I + 1 & " , " & Format(CDbl(I) / CDbl(EnemyCount) * 100, "00") & "%")
                    AgateLib.DisplayLib.Display.FillRect(0, GameDisplayObject.MyDisplayHeight / 2, CDbl(I) / CDbl(EnemyCount) * GameDisplayObject.MyDisplayWidth, 20, New AgateLib.Geometry.Gradient(AgateLib.Geometry.Color.Maroon, AgateLib.Geometry.Color.Red, AgateLib.Geometry.Color.Orange, AgateLib.Geometry.Color.Yellow))
                    AgateLib.DisplayLib.Display.EndFrame()
                End If

                ' give some time so that the application does not get blocked
                Application.DoEvents()

                ' load the creature
                Dim CreatureIdentificationByte As Integer
                CreatureIdentificationByte = FS.ReadByte
                Dim CreatureObject As BaseCreature = Nothing


                ' loading stupid enemy
                If CreatureIdentificationByte = 0 Then CreatureObject = New StupidEnemy

                ' loading presistant enemy
                If CreatureIdentificationByte = 1 Then CreatureObject = New PresistantEnemy

                ' loading monkey enemy
                If CreatureIdentificationByte = 2 Then CreatureObject = New MonkeyEnemy

                ' loading smoke effect
                If CreatureIdentificationByte = 3 Then CreatureObject = New Smoke

                ' loading information bubble
                If CreatureIdentificationByte = 4 Then
                    CreatureObject = New Info
                    Dim TX As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B(0 To TX - 1) As Byte
                    FS.Read(B, 0, B.Length)
                    Dim Tmp As Info = CreatureObject
                    Tmp.Message = System.Text.Encoding.GetEncoding("windows-1256").GetString(B)
                End If

                ' loading spider enemy
                If CreatureIdentificationByte = 5 Then CreatureObject = New SpiderEnemy

                ' loading on off button
                If CreatureIdentificationByte = 6 Then
                    CreatureObject = New OnOffButton
                    Dim TX As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B(0 To TX - 1) As Byte
                    FS.Read(B, 0, B.Length)
                    Dim Tmp As OnOffButton = CreatureObject
                    Tmp.ObjectToAffact = System.Text.Encoding.GetEncoding("windows-1256").GetString(B)
                End If

                ' loading gate
                If CreatureIdentificationByte = 7 Then
                    CreatureObject = New Gate
                    Dim TX As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B(0 To TX - 1) As Byte
                    FS.Read(B, 0, B.Length)
                    Dim Tmp As Gate = CreatureObject
                    Tmp.ObjectName = System.Text.Encoding.GetEncoding("windows-1256").GetString(B)
                End If

                ' loading action point
                If CreatureIdentificationByte = 8 Then
                    CreatureObject = New Action
                    Dim TX As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B(0 To TX - 1) As Byte
                    FS.Read(B, 0, B.Length)
                    Dim Tmp As Action = CreatureObject
                    Tmp.ActionType = System.Text.Encoding.GetEncoding("windows-1256").GetString(B)
                End If

                ' loading wolfman enemy
                If CreatureIdentificationByte = 9 Then CreatureObject = New WolfmanEnemy

                ' loading teleport point
                If CreatureIdentificationByte = 10 Then
                    CreatureObject = New Teleport
                    Dim TX As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B(0 To TX - 1) As Byte
                    FS.Read(B, 0, B.Length)
                    Dim Tmp As Teleport = CreatureObject
                    Tmp.ObjectName = System.Text.Encoding.GetEncoding("windows-1256").GetString(B)
                    Dim TX2 As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B2(0 To TX2 - 1) As Byte
                    FS.Read(B2, 0, B2.Length)
                    Tmp.TeleportTo = System.Text.Encoding.GetEncoding("windows-1256").GetString(B2)
                End If

                ' loading bat enemy
                If CreatureIdentificationByte = 11 Then CreatureObject = New BatEnemy

                ' loading push button
                If CreatureIdentificationByte = 12 Then
                    CreatureObject = New PushButton
                    Dim TX As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B(0 To TX - 1) As Byte
                    FS.Read(B, 0, B.Length)
                    Dim Tmp As PushButton = CreatureObject
                    Tmp.ObjectToAffact = System.Text.Encoding.GetEncoding("windows-1256").GetString(B)
                End If

                ' loading push button gate
                If CreatureIdentificationByte = 13 Then
                    CreatureObject = New Gate2
                    Dim TX As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B(0 To TX - 1) As Byte
                    FS.Read(B, 0, B.Length)
                    Dim Tmp As Gate2 = CreatureObject
                    Tmp.ObjectName = System.Text.Encoding.GetEncoding("windows-1256").GetString(B)
                End If

                ' loading arrow button
                If CreatureIdentificationByte = 14 Then
                    CreatureObject = New ArrowButton
                    Dim TX As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B(0 To TX - 1) As Byte
                    FS.Read(B, 0, B.Length)
                    Dim Tmp As ArrowButton = CreatureObject
                    Tmp.ObjectToAffact = System.Text.Encoding.GetEncoding("windows-1256").GetString(B)
                End If

                ' loading arrow source
                If CreatureIdentificationByte = 15 Then
                    Dim DRN = FS.ReadByte
                    CreatureObject = New ArrowSource(DRN)
                    Dim TX As Long = FS.ReadByte + FS.ReadByte * 256
                    Dim B(0 To TX - 1) As Byte
                    FS.Read(B, 0, B.Length)
                    Dim Tmp As ArrowSource = CreatureObject
                    Tmp.ObjectName = System.Text.Encoding.GetEncoding("windows-1256").GetString(B)
                End If

                ' load the x and y for the creature
                CreatureObject.MapX = FS.ReadByte + FS.ReadByte * 256
                CreatureObject.MapY = FS.ReadByte + FS.ReadByte * 256

                GameCreaturesObject.AllCreatureList.Add(CreatureObject)
            Next


            ' load the boxes
            Dim BoxCount = FS.ReadByte()
            For I = 0 To BoxCount - 1

                ' display progress
                If RenderProgress Then
                    AgateLib.DisplayLib.Display.BeginFrame()
                    AgateLib.DisplayLib.Display.Clear(0, 0, 0, 0)
                    GameTextSufaceObject.DrawText(0, 0, "5 : loading box no:" & I + 1 & " , " & Format(CDbl(I) / CDbl(EnemyCount) * 100, "00") & "%")
                    AgateLib.DisplayLib.Display.FillRect(0, GameDisplayObject.MyDisplayHeight / 2, CDbl(I) / CDbl(BoxCount) * GameDisplayObject.MyDisplayWidth, 20, New AgateLib.Geometry.Gradient(AgateLib.Geometry.Color.Maroon, AgateLib.Geometry.Color.Red, AgateLib.Geometry.Color.Orange, AgateLib.Geometry.Color.Yellow))
                    AgateLib.DisplayLib.Display.EndFrame()
                End If
                Dim X1 As Integer
                Dim X2 As Integer
                Dim X3 As Integer
                X1 = FS.ReadByte
                X2 = FS.ReadByte + FS.ReadByte * 256
                X3 = FS.ReadByte + FS.ReadByte * 256
                Dim B As New Box(X1, X2, X3)
                GameCreaturesObject.BoxList.Add(B)
            Next

            FS.Close()

            ' update the game map information and numbers
            GameMap = M
            GameMapActualWidth = GameMap.GetLength(0) * 16
            GameMapActualHeight = GameMap.GetLength(1) * 16
            GameMapWidthDiv16 = GameMap.GetLength(0)
            GameMapHeightDiv16 = GameMap.GetLength(1)

            ' load the wall paper for this map
            If System.IO.File.Exists(System.IO.Path.GetDirectoryName(FileName) & "\" & System.IO.Path.GetFileNameWithoutExtension(FileName) & ".jpg") Then

                ' first load the image
                Wallpaper.WallpaperImg = New AgateLib.DisplayLib.Surface(System.IO.Path.GetDirectoryName(FileName) & "\" & System.IO.Path.GetFileNameWithoutExtension(FileName) & ".jpg")

                ' if the image is smaller than the display dimensions, resize it to fit.
                If Wallpaper.WallpaperImg.DisplayWidth < GameDisplayObject.MyDisplayWidth Then
                    Wallpaper.WallpaperImg.DisplaySize = New AgateLib.Geometry.Size(GameDisplayObject.MyDisplayWidth, GameDisplayObject.MyDisplayHeight)
                End If

                ' next start calculating the display change ration mapped to the viewx, and viewy
                ' start with the part of the map you want to see. Notice that the display width and hight
                ' should be subtracted so that you don't get missing parts of the image at the end of the map
                Wallpaper.WallpaperWidth = Wallpaper.WallpaperImg.DisplayWidth - GameDisplayObject.MyDisplayWidth
                Wallpaper.WallpaperHeight = Wallpaper.WallpaperImg.DisplayHeight - GameDisplayObject.MyDisplayHeight


                ' set the flag for using the wall paper
                Wallpaper.UseWallpaper = True

                ' calculate the size of the map in pixel so that we could use it to determine the ratio
                Dim MapWidthInPixel As Double = M.GetLength(0) * 16
                Dim MapHeightInPixel As Double = M.GetLength(1) * 16

                ' calculate the ratio
                Wallpaper.WallpaperXRatio = (CDbl(Wallpaper.WallpaperWidth) / MapWidthInPixel)
                Wallpaper.WallpaperYRatio = (CDbl(Wallpaper.WallpaperHeight) / MapHeightInPixel)

                ' now check for the case when the resolution of the map > the size of map
                If Wallpaper.WallpaperXRatio > 1 Then Wallpaper.WallpaperXRatio = 1
                If Wallpaper.WallpaperYRatio > 1 Then Wallpaper.WallpaperYRatio = 1

                ' check for the case when the size of the map = size of display
                If Wallpaper.WallpaperXRatio = 0 Then Wallpaper.WallpaperWidth = Wallpaper.WallpaperImg.DisplayWidth
                If Wallpaper.WallpaperYRatio = 0 Then Wallpaper.WallpaperHeight = Wallpaper.WallpaperImg.DisplayHeight

            Else
                Wallpaper.UseWallpaper = False
            End If

            Dim MapMusicFileName As String = System.IO.Path.GetDirectoryName(FileName) & "\" & System.IO.Path.GetFileNameWithoutExtension(FileName) & ".mp3"
            If System.IO.File.Exists(MapMusicFileName) Then
                If RenderProgress Then
                    AgateLib.DisplayLib.Display.BeginFrame()
                    AgateLib.DisplayLib.Display.Clear(0, 0, 0, 0)
                    GameTextSufaceObject.DrawText(0, 0, "6 : laoding music")
                    AgateLib.DisplayLib.Display.EndFrame()
                End If
                MapMusic = New Microsoft.DirectX.AudioVideoPlayback.Audio(MapMusicFileName)
                MapMusic.Volume = 0.5
                MapMusic.Play()
                MapMusicPosition = -1
            Else
                MapMusic = Nothing
            End If

            GameCreaturesObject.FillListsWithCorrectTypeOfCreature()
        Catch ex As Exception

            MsgBox(ex.Message)
        End Try
    End Sub


End Class
