﻿Public Class Arrow
    Inherits BaseCreature

    Public Enum ArrowDirection
        ArrowDirectionUp = 0
        ArrowDirectionDown = 1
        ArrowDirectionLeft = 2
        ArrowDirectionRight = 3
    End Enum

    ' here we define a number of properties for our hero
    Public ArrowShootTime As Long

    Public Sub New(ByVal X As Integer, ByVal Y As Integer, ByVal AD As ArrowDirection)
        IsRendered = True
        MapX = X
        MapY = Y
        RenderStage = 0
        If AD = ArrowDirection.ArrowDirectionDown Or AD = ArrowDirection.ArrowDirectionUp Then
            CreatureWidth = 16
            CreatureHeight = 32
        Else
            CreatureWidth = 32
            CreatureHeight = 16
        End If

        If AD = ArrowDirection.ArrowDirectionDown Then AccX = 0 : AccY = 6
        If AD = ArrowDirection.ArrowDirectionLeft Then AccX = -6 : AccY = 0
        If AD = ArrowDirection.ArrowDirectionRight Then AccX = 6 : AccY = 0
        If AD = ArrowDirection.ArrowDirectionUp Then AccX = 0 : AccY = -6


        HalfWidth = CreatureWidth \ 2 - 0
        HalfHeight = CreatureHeight \ 2 - 0
        IsAffectedByGravity = False
        CanBounce = False
        Gravity = 0.1
        MaxGravity = 10
        SolidBlocks = New List(Of MapBlockType)
        SolidBlocks.Add(MapBlockType.BrickBlock)
        SolidBlocks.Add(MapBlockType.RocksBlock)
        SolidBlocks.Add(MapBlockType.GrassBlock)
        SolidBlocks.Add(MapBlockType.BreakableBrick)

        ' load the dummy shapes
        If AD = ArrowDirection.ArrowDirectionDown Then CreatureSprites = GameSpritesObject.ArrowDownSprite
        If AD = ArrowDirection.ArrowDirectionUp Then CreatureSprites = GameSpritesObject.ArrowUpSprite
        If AD = ArrowDirection.ArrowDirectionLeft Then CreatureSprites = GameSpritesObject.ArrowLeftSprite
        If AD = ArrowDirection.ArrowDirectionRight Then CreatureSprites = GameSpritesObject.ArrowRightSprite
        CreatureSprites.CurrentFrameIndex = 0
        ArrowShootTime = System.Environment.TickCount
    End Sub

    ' for future update
    Public Overrides Sub UpdateLocation(ByVal Info As Object, ByVal UserInput As GameInputClass)
        If Not Me.IsRendered Then
            Exit Sub
        End If

        Dim X16 As Integer = Me.MapX \ 16
        Dim Y16 As Integer = (Me.MapY - Me.HalfHeight) \ 16

        If X16 < 0 Or X16 >= GameMapObject.GameMap.GetLength(0) Then
            Me.IsRendered = False
            Return
        End If

        If Y16 < 0 Or Y16 >= GameMapObject.GameMap.GetLength(1) Then
            Me.IsRendered = False
            Return
        End If

        For Each I In SolidBlocks
            If GameMapObject.GameMap(X16, Y16) = I Then
                Me.IsRendered = False
                Return
            End If
        Next

        Dim Z As Single = GameTimeObject.GameTimeDiff
        Me.MapY = Z * AccY + MapY
        Me.MapX = Z * AccX + MapX

        If System.Environment.TickCount - ArrowShootTime > 10000 Then
            Me.IsRendered = False
        End If

        If AccX = 0 And AccY = 0 Then
            Me.IsRendered = False
        End If

    End Sub



End Class
