﻿Public Class IgnitedBouncingBomb
    Inherits BaseCreature

    Public Enum DynamiteState
        DynamiteStillIgnited = 0
        DynamiteExploding = 1
    End Enum

    ' here we define a number of properties for our hero
    Public DynamiteMode As DynamiteState
    Public ExplodingCounter As Integer = 100
    Public Exploded As Boolean = False
    Private PixelListX As New List(Of Single)
    Private PixelListY As New List(Of Single)
    Private PixelListXX As New List(Of Single)
    Private PixelListYY As New List(Of Single)
    Private PixelColor As New List(Of AgateLib.Geometry.Color)
    Private OldYValue As Integer = 0
    Private OldTime As Integer = 0

    Public Sub New(ByVal X As Integer, ByVal Y As Integer, ByVal AccX As Double, ByVal AccY As Double)
        DynamiteMode = DynamiteState.DynamiteStillIgnited
        IsRendered = True
        MapX = X
        MapY = Y
        Me.AccX = AccX
        Me.AccY = AccY
        RenderStage = 0
        CreatureWidth = 16
        CreatureHeight = 16
        HalfWidth = CreatureWidth \ 2 - 0
        HalfHeight = CreatureHeight \ 2 - 0
        IsAffectedByGravity = True
        CanBounce = True
        Gravity = 0.1
        MaxGravity = 10
        SolidBlocks = New List(Of MapBlockType)
        SolidBlocks.Add(MapBlockType.BrickBlock)
        SolidBlocks.Add(MapBlockType.RocksBlock)
        SolidBlocks.Add(MapBlockType.GrassBlock)
        SolidBlocks.Add(MapBlockType.BreakableBrick)

        Me.IsStanding = False

        ' load the dummy shapes
        CreatureSprites = GameSpritesObject.IgnitedBombSprite
        CreatureSprites.CurrentFrameIndex = 0

    End Sub

    ' for future update
    Public Overrides Sub UpdateLocation(ByVal Info As Object, ByVal UserInput As GameInputClass)

        ' checking for when to explode
        Dim Tmp As Integer = MapY - OldYValue
        If Tmp < 0 Then Tmp = -Tmp

        If Tmp < 2 Then
            Dim Tmp2 As Integer = AccY
            If Tmp2 < 0 Then Tmp2 = -Tmp2
            If Tmp2 < 0.5 Then
                If OldTime = 0 Then
                    OldTime = System.Environment.TickCount
                ElseIf System.Environment.TickCount - OldTime > 1000 Then
                    CanBounce = False
                End If
            End If
        Else
            OldTime = 0
        End If
        OldYValue = MapY

        If Not Me.IsRendered Then
            Exit Sub
        End If
        If Not Me.IsStanding Then
            MyBase.UpdateLocation(Info, UserInput)


            Exit Sub
        End If
        If Me.DynamiteMode = DynamiteState.DynamiteStillIgnited Then
            Me.IsAffectedByGravity = False
            DynamiteMode = DynamiteState.DynamiteExploding
            Dim I As Integer
            Dim R As Random = GameRandomNumberObject
            For I = 0 To 99
                PixelListX.Add(Me.MapX)
                PixelListY.Add(Me.MapY)
                PixelListXX.Add(2 - R.NextDouble * 4)
                PixelListYY.Add(2 - R.NextDouble * 4)
                PixelColor.Add(AgateLib.Geometry.Color.FromArgb(R.Next(0, 255), 255, R.Next(0, 50), R.Next(0, 50)))
            Next
            Exploded = True
            Exit Sub
        ElseIf Me.IsStanding Then
            ExplodingCounter = ExplodingCounter - 1
            If ExplodingCounter = 0 Then
                Me.IsRendered = False
            End If
        End If
    End Sub


    Public Overrides Sub RenderCreature()
        If IsRendered Then
            If DynamiteMode = DynamiteState.DynamiteStillIgnited Then
                If AccX > 0 Then
                    CreatureSprites.RotationAngle = -((System.Environment.TickCount) Mod 360) / 180 * Math.PI
                Else
                    CreatureSprites.RotationAngle = ((System.Environment.TickCount) Mod 360) / 180 * Math.PI
                End If

                MyBase.RenderCreature()
            Else

                Dim I As Integer
                For I = 0 To PixelListX.Count - 1
                    PixelListX(I) += PixelListXX(I)
                    PixelListY(I) += PixelListYY(I)
                    AgateLib.DisplayLib.Display.FillRect(PixelListX(I) - GameMapObject.ViewX1 + GameMapObject.ViewXShift - Me.HalfWidth, PixelListY(I) - GameMapObject.ViewY1 + GameMapObject.ViewYShift - Me.HalfHeight, 2, 2, PixelColor(I))
                Next

            End If

        End If
    End Sub


End Class
